module ApplicationHelper

  def button_class(class_name)
    class_name_to_button_class = {
      'awsaccounts' => 'btn btn-launch spdb-button aws-button',
      'federations' => 'btn btn-launch spdb-button aws-button',
      'idps'        => 'btn btn-launch spdb-button idps-button',
    }

    return class_name_to_button_class[class_name.downcase()]
  end

  def field_label(field, text)
    tool_tip = help_tool_tip(field)
    result = "<strong >#{text} #{tool_tip}:</strong>"
    return result
  end

  def remove_line_feeds(text)
    if (text.nil?())
      return text
    else
      return text.gsub(/\n/, ' ')
    end
  end

  def help_tool_tip(tag_name)
    message = help_for_tag(tag_name)
    message = remove_line_feeds(message)
    span_element = %Q[<span title="#{message}"> [?]</span>]
    return span_element
  end

  def help_for_tag(tag_name)
    check_valid_until_expired_text = Rails.configuration.check_valid_until_expired_text
    #
    help_messages = {
     :metadata => <<-eoa,
The Service Provider's metadata. This must be a valid SAML 2.0 XML
string.
This is a REQUIRED field.
eoa
     :entityid => <<-eoa,
The Service Provider's entityId. This is usually a URL, but can
be any string not used by any other SP.
eoa
     :login_url => <<-eoa,
The URL used to login to the protected application.
eoa
     :contact_email => <<-eob,
A Mailman address or other group-based e-mail address.
Notifications about system issues, certificate expirations, or other
technical and administrative questions will be sent to this address.
This is a REQUIRED field.
eob
     :comments => <<-eob,
Provide short description of your application and/or any specific
configuration requirements. This is a required field.
eob
     :owned_by => <<-eob,
The SUNet ID of the user who created this entry. Only this user will be
able to see or modify this entry.
eob
     :sign_responses => <<-eob,
If the Service Provider wants the Identity Provider's responses to be
signed, this box must be checked. If you are unsure whether to check this
box or not, contact your Service Provider's technical support.
eob
     :encrypt_assertions => <<-eob,
If the Service Provider wants the Identity Provider's assertions to be
encrypted, this box must be checked. If you are unsure whether to check this
box or not, contact your Service Provider's technical support.
eob
     :sign_assertions => <<-eob,
If the Service Provider wants the Identity Provider's assertions to be
signed, this box must be checked. If you are unsure whether to check this
box or not, contact your Service Provider's technical support.
eob
     :created_at => <<-eob,
The date and time this record was created.
eob
     :updated_at => <<-eob,
The date and time this record was last changed.
eob
     :created_by => <<-eob,
The user who created this record.
eob
     :updated_by => <<-eob,
The user who last changed this record.
eob
     :owned_by => <<-eob,
The user who owns this record. This atrribute is ADMIN viewable only.
eob
     :acs_endpoint => <<-eob,
Assertion Consumer Service Endpoints - These are the locations to which the
IdP will eventually send the user at the Service Provider.
eob
     :workgroup => <<-eob,
Add or remove Stanford Workgroups that have privileges to manage this
resource. You can see a list of workgroups you belong to at the
Workgroup Manager web site https://workgroup.stanford.edu/
eob
     :is_legacy => <<-eob,
If true this is a "legacy" configuration, that is, one added before the
blanket release policy. This field is editable only by SPDB admins.
eob
     :cert => <<-eob,
Expiration and usage information about the certificates in your
metadata. It is best practice to use a self-signed certificate that
expires in no sooner than four years.
eob
     :valid_until => <<-eob,
Service Provider metadata should have a validUntil attribute that
indicates when the metadata is no longer valid.
After the validUntil date the IdP will not recognize your SP's metadata.
If used the validUntil attribute must be set to
a date at least #{check_valid_until_expired_text} into the future.
eob
     :instance => <<-eob,
The AWS instance these mappings belong to. Will usually be "AWS" but
could also be "GovCloud".
eob
     :account_number => <<-eob,
The AWS account number. Must be exactly 12 digits.
eob
     :account_master => <<-eob,
If this account is a sub-account, put the master account number in this
field.
eob
     :account_nickname => <<-eob,
A short name to help identify this AWS account.
eob
     :id_provider => <<-eob,
The AWS SAML Identity Provider. This is automatically derived from the
account number.
eob
     :mappings => <<-eob,
These are the Stanford Workgroup to AWS IAM mappings. Each one consists
of the pair (workgroup, AWS role ARN).
eob
     :awsaccount_notes => <<-eob,
Any extra notes concerning this AWS account, for example, who are the
primary administrators.
eob
     :ignore_cert_issues => <<-eob,
Ignore certificate issues (expired, missing, etc.) when saving the configuration.
This option is only available to SPDB administrators.
eob
     :ignore_valid_until_issues => <<-eob,
Ignore validUntil attribute issues (expired, missing, etc.) when saving the configuration.
This option is only available to SPDB administrators.
eob
     :workgrouptorole => <<-eob,
The mapping of a Stanford Workgroup to an AWS IAM role.
The first element is a Stanford Workgroup name. The second element is
the ARN of an AWS IAM role. The third element is optional and is for any
extra notes you may want to add concerning this mapping.
eob
     :idps_entityid => <<-eob,
The entity Id of this Identity Provider.
eob
     :idps_metadata => <<-eob,
The IdP SAML 2.0 metadata for this Identity Provider.
eob
     :idps_cert => <<-eob,
Information on all certificates contained in this IdPs metadata.
eob
     :idps_url => <<-eob,
The URL where this IdP metadata can be downloaded.
eob
     :idps_comments => <<-eob,
Any extra information for this IdP.
eob
     :idps_account_nickname => <<-eob,
A short description of this AWS account.
eob
    }
    return help_messages[tag_name]
  end
end

def utc_to_local_time(utc_time)
  utc_time.in_time_zone("Pacific Time (US & Canada)")
end
