namespace :notifications do

  desc "task to generate e-mail notifications"
  task :mails => :environment do

    emails  = {}

    # Get all the spconfigs grouped by contact_email
    query = 'SELECT entityid, contact_email FROM spconfigs'
    results = ActiveRecord::Base.connection.execute(query)
    results.each do |result|
      entityid      = result[0]
      contact_email = result[1]
      puts "contact_email is [#{contact_email}]"

      # If there is no contact_email, skip
      if (contact_email.nil? || contact_email.empty?) then
        puts "no contact_email so skipping #{entityid}"
        next
      else
        if (emails.key?(contact_email)) then
          emails[contact_email].push(entityid)
        else
          emails[contact_email] = [entityid]
        end
      end
    end

    emails.each do |contact_email, entityids|
      result = NotificationMailer.testing_notification_email(contact_email, entityids, false).deliver()
      puts "result: #{result.inspect()}"
    end
  end

  ######################################################################
  #
  # Generate an e-mail if a certificate is set to expire.
  #
  # From command line, to run with day_window set to 30 days:
  #
  #    bin/rake notifications:cert_expirations[30] RAILS_ENV=development
  #
  # or
  #
  #    bin/rake notifications:cert_expirations[30]
  #
  # To override e-mail:
  #
  #    bin/rake notifications:cert_expirations[30,adamhl@stanford.edu]
  #
  # To specify the environment:
  #
  #    RAILS_ENV=production bin/rake notifications:cert_expirations[30,adamhl@stanford.edu]

  desc "task to generate e-mail notifications of certificate expirations"
  task :cert_expirations, [:day_window,:email_override] => :environment do |t, args|

    day_window = args[:day_window]
    if (day_window.nil?()) then
      raise "missing argument for day_window"
    else
      day_window = args['day_window'].to_i
    end

    email_override = args[:email_override]

    now_utc            = Time.now.utc
    warning_date       = now_utc + day_window.days
    warning_date_plus1 = now_utc + (day_window + 1).days

    spconfigs = Spconfig.all()

    spconfigs.each do |spconfig|
      certificates = spconfig.certificates

      found_expiring_certificate = false
      certificates.each do |certificate|
        # Get expiration date
        if ((certificate.expires_at < warning_date_plus1) and
            (certificate.expires_at >= warning_date)) then
          found_expiring_certificate = true
        end
      end
      if (found_expiring_certificate) then
        # puts "one or more certificates associated with #{spconfig.entityid} are expiring in #{day_window} days"
        result = CertExpirationMailer.warning_email(spconfig:       spconfig,
                                                    day_window:     day_window,
                                                    email_override: email_override).deliver()
        # puts "result: #{result.inspect()}"
      end
    end

  end

  ######################################################################
  #
  # Generate an e-mail if an SP configuration is set to expire (based on
  # validUntil).
  #
  # From command line, to run with day_window set to 30 days:
  #
  #    bin/rake notifications:spconfig_expirations[30] RAILS_ENV=development
  #
  # or
  #
  #    bin/rake notifications:spconfig_expirations[30]
  #
  # To override e-mail:
  #
  #    bin/rake notifications:spconfig_expirations[30,adamhl@stanford.edu]
  #
  # To specify the environment:
  #
  #    RAILS_ENV=production bin/rake notifications:spconfig_expirations[30,adamhl@stanford.edu]

  desc "task to generate e-mail notifications of SP configuration expirations"
  task :spconfig_expirations, [:day_window,:email_override] => :environment do |t, args|

    day_window = args[:day_window]
    if (day_window.nil?()) then
      raise "missing argument for day_window"
    else
      day_window = args['day_window'].to_i
    end

    email_override = args[:email_override]

    now_utc            = Time.now.utc
    warning_date       = now_utc + day_window.days
    warning_date_plus1 = now_utc + (day_window + 1).days

    spconfigs = Spconfig.all()

    spconfigs.each do |spconfig|
      valid_until = spconfig.valid_until

      # Skip sp configurations with blank validUntil's.
      if (valid_until.blank?()) then
        next
      end

      if ((valid_until < warning_date_plus1) and
            (valid_until >= warning_date)) then
        result = SpconfigExpirationMailer.warning_email(spconfig:       spconfig,
                                                        day_window:     day_window,
                                                        email_override: email_override).deliver()
        # puts "result: #{result.inspect()}"
      end
    end
  end


  # Generate list of configs along with their expirations
  desc "task to generate list of configs with their expirations"
  task :list_expirations do
    certificates = Certificate.all().order(:expires_at)
    certificates.each do |certificate|
      spconfig = certificate.spconfig
      use = certificate.use
      puts spconfig.entityid
      puts "  #{certificate.expires_at.to_s()} (#{use})"
    end
  end


end
