#!/usr/bin/python
import traceback
from time import sleep
from xml.sax.saxutils import escape, quoteattr
import os, sys, unittest, time, re, random, string, tempfile
from datetime import datetime
from datetime import timedelta
from subprocess import call
import argparse
import inspect
import tempfile
import pprint

# In case selenium module is not installed.
try:
  from selenium import webdriver
  from selenium.webdriver.chrome.options import Options
  from selenium.webdriver.common.by import By
  from selenium.webdriver.common.keys import Keys
  from selenium.webdriver.common.action_chains import ActionChains
  from selenium.webdriver.support.ui import Select
  from selenium.webdriver.support.ui import WebDriverWait
  from selenium.webdriver.support import expected_conditions as EC
  from selenium.common.exceptions import UnexpectedAlertPresentException
  from selenium.common.exceptions import StaleElementReferenceException
  from selenium.common.exceptions import NoAlertPresentException
  from selenium.common.exceptions import NoSuchElementException
  from selenium.webdriver.common.alert import Alert
except ImportError:
  pass

# Note: Remember that WebDriverWait(driver, 30).until(some_method)
# repeatedly calls "some_method(driver)" until that call returns true, or
# else the time runs out.

# If the document is in the "complete" state (i.e., the document has
# completed loading) return true, otherwise return false.
def readystate_complete(driver):
  return driver.execute_script("return document.readyState") == "complete"

class SPDBTesting(object):

  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # Set up the webdriver object. To access it, use "self.driver".
  def __init__(self, environment, browser, username = 'unknown', verbose = False):

    self.environment = environment.lower()
    self.browser     = browser
    self.username    = username
    self.password    = 'spdb-quality-assurance'
    self.awsaccount  = '012345678999'
    self.url         = None

    self.verbose     = verbose
    self.pprint      = pprint.PrettyPrinter(indent=4)

    if (browser == 'chrome'):
      if (sys.platform.lower() == 'win32'):
        chromedriver = "D:\\bin\\chromedriver.exe"
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chromedriver)
      else:
        chromedriver = "/usr/sbin/chromedriver"
        chrome_options = Options()
        chrome_options.add_argument("--disable-infobars")
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chrome_options=chrome_options)
    elif (browser == 'firefox'):
      self.driver = webdriver.Firefox()
    elif (browser == 'phantomjs'):
      self.driver = webdriver.PhantomJS(service_args=['--ignore-ssl-errors=true'])
      self.driver.set_window_size(1120, 550)
    else:
      raise ValueError("unknown browser " + browser)

    self.fake_wait   = True
    self.test_string = None
    self.driver.maximize_window()
    self.driver.implicitly_wait(30)
    self.use_basic_auth = True
    self.hostname = 'spdb-' + environment + '.stanford.edu'
    self.set_base_url()
    self.verificationErrors = []
    self.accept_next_alert = True

  def progress(self, message):
    if (self.verbose):
      if (message is None):
        message = 'None'
      print('progress: ' + message)

  def pp(self, object):
    return self.pprint.pformat(object)

  def exit_with_exception(self, message):
    page_source = self.driver.page_source.encode('utf-8').strip()

    fp = open('out.html', 'w')
    fp.write(page_source)
    fp.close()

    self.driver.save_screenshot('out.png');

    self.driver.quit();
    raise Exception(message + "; see screen shot in out.png and page source is in out.html")

  # When running a workgroup membership test we need to have
  # a real user (e.g., "adamhl", "cheroux", "vivienwu", etc.). Note
  # that when we set the username the url changes as well.
  def reset_username(self, user_name):
    self.username = user_name
    self.set_base_url()

  # Set the url attribute. This will depend on the authentication
  # mechanism and the user
  def set_base_url(self):
    if (self.use_basic_auth):
      url = "https://{}:{}@{}/{}".format(self.username, self.password, self.hostname, self.url)
    else:
      url = "https://{}/{}".format(self.hostname, self.url)

    self.base_url = url
    return self.base_url

  def go_home(self):
    return self.driver.get(self.base_url)

  # Get the HTML that comprises the red error text (flash) box at the top
  # of the screen. If there is no such box return None.
  def get_red_error_text(self):
    box_id = 'error_explanation'
    error_box_element = None
    try:
      error_box_element = self.driver.find_element_by_id(box_id)
    except NoSuchElementException, e:
      # No such element, so return None.
      return None

    return error_box_element.get_attribute('innerHTML')

  # Get the HTML that comprises the green text (flash) box at the top
  # of the screen. If there is no such box return None.
  def get_message_text(self):
    box_id = 'notice_box'
    message_box_element = None
    try:
      message_box_element = self.driver.find_element_by_class_name(box_id)
    except NoSuchElementException, e:
      # No such element, so return None.
      return None

    return message_box_element.get_attribute('innerHTML')

  # Use send_keys on an element but only chunk_size characters at a time.
  def send_keys_slow(self, element, text, chunk_size=128):
    while (len(text) > 0):
      current_chunk = text[0:chunk_size]
      element.send_keys(current_chunk)
      text = text[chunk_size:]

  def get_element_by_id(self, id):
    return self.driver.find_element_by_id(id)

  #
  # Use the search function to find one or more SP configs.
  #
  def search_for_aws(self, search_text):
    driver = self.driver

    self.go_home()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("search").clear()
    driver.find_element_by_id("search").send_keys(search_text)
    driver.find_element_by_id("button_spconfig_search").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    if ("Search found 0 AWS" in driver.page_source):
      return 0
    else:
      return 1

  #
  # Assuming we are on the view page and there is only ONE SP configuration listed,
  # click the update icon.
  #
  def click_update_icon(self):
    try:
      update_link = self.driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]")
    except:
      self.exit_with_exception("could not find update icon using css selector")

    update_link.click()

  @staticmethod
  def parse_args():
    parser = argparse.ArgumentParser(description='Run Selenium tests')
    parser.add_argument('--environment', help='SPDB environment (PROD, STAGE, UAT, DEV)',
                        )
    parser.add_argument('--browser', help='browser model',
                        choices=['chrome', 'firefox', 'phantomjs'])
    parser.add_argument('--entityID', help='explicitly specify the entityID'
                        )
    parser.add_argument('--tests', help='comma-separated list of tests to run (or ALL)'
                        )
    parser.add_argument('--verbose', action='store_true',
                        help='be verbose'
                        )
    return parser.parse_args()

  @staticmethod
  def assertRegexpMatches(ut, text, expected_regexp, msg=None):
    try:
      unittest.TestCase.assertRegexpMatches(ut, text, expected_regexp, msg)
    except Exception, e:
      # print text
      raise ut.failureException(msg or "Regexp \"" + expected_regexp + "\" didn't match")

class SPDBUnitTest(unittest.TestCase):

  ENVIRONMENT        = 'DEV'
  BROWSER            = 'chrome'
  GENERATE_METADATA  = False
  USE_METADATA_FILES = False
  SPDB               = None
  USERNAME           = 'unknown'
  TESTS              = 'ALL'
  VERBOSE            = False

  # Workgroup settings
  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # WG_USER1 belongs to workgroup WG1. Note that WG_USER1 is an admin, so
  # be sure to run some of the tests as a non-admin.
  WG_USER1 = 'cheroux'

  # WG_USER2 does NOT belong to workgroup WG1. WG_USER2 is NOT an admin.
  WG_USER2 = 'jheroux'

  # WG_USER3 belongs to workgroup WG1.
  WG_USER3 = 'rraman'

  # ADMIN users
  WG_ADMIN   = 'cheroux'

  # REGULAR user
  WG_REGULAR = 'jheroux'

  @classmethod
  def setUpClass(cls):
    print "\nRunning " + sys.argv[0] + " in environment " + cls.ENVIRONMENT + "\n"
    cls.SPDB = SPDBTesting(environment = cls.ENVIRONMENT,
                           browser     = cls.BROWSER,
                           verbose     = cls.VERBOSE
    )

  def setUp(self):
    pass

  def progress(self, message):
    if (self.VERBOSE):
      print('progress: ' + message)

  # Call this method before running any workgroup test.
  def set_username(self, user_name):
    self.SPDB.reset_username(user_name)
    return

  def get_page_source(self):
    return self.SPDB.driver.page_source.encode('utf-8').strip()

  def test_311_check_awsaccounts_page(self):
    print "." + inspect.stack()[0][3] + " --> ISHIB-311"

    self.SPDB.url = 'home'
    self.set_username(self.WG_USER1)
    sys.stdout.write("\tGoing to /home page (with admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'Manage AWS account mappings' in source:
        print "SUCCESS! Manage AWS account mappings button appeared for admin user"
      else:
        print "FAILURE! Manage AWS account mappings button did not appear for admin user"
    except NoSuchElementException, e:
      return None

    self.SPDB.url = 'home'
    self.set_username(self.WG_USER2)
    sys.stdout.write("\tGoing to /home page (with non-admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'Manage AWS account mappings' in source:
        print "FAILURE! Manage AWS account mappings button appeared for non-admin user"
      else:
        print "SUCCESS! Manage AWS account mappings button did not appear for non-admin user"
    except NoSuchElementException, e:
      return None

  def test_312_check_awsaccounts_page(self):
    print "." + inspect.stack()[0][3] + " --> ISHIB-312"

    self.SPDB.url = 'awsaccounts'
    self.set_username(self.WG_USER1)
    sys.stdout.write("\tGoing to /awsaccounts page (with admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'button_awsaccounts_all' in source:
        print "SUCCESS! List All button appeared for admin user"
      else:
        print "FAILURE! All button did not appear for admin user"
    except NoSuchElementException, e:
      return None

    self.SPDB.url = 'awsaccounts'
    self.set_username(self.WG_USER2)
    sys.stdout.write("\tGoing to /awsaccounts page (with non-admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'Forbidden' in source:
        print "SUCCESS! Forbidden message appeared for non-admin user"
      else:
        print "FAILURE! Forbidden message not returned non-admin user"
    except NoSuchElementException, e:
      return None

  def test_313_check_awsaccounts_page(self):
    print "." + inspect.stack()[0][3] + " --> ISHIB-313"

    test_pages = [
      ["new", "admin", self.WG_USER1, "Account Number"],
      ["new", "non-admin", self.WG_USER2, "Forbidden"],
      ["list", "admin", self.WG_USER1, "Account Number"],
      ["list", "non-admin", self.WG_USER2, "Forbidden"],
      ["all", "admin", self.WG_USER1, "Account Number"],
      ["all", "non-admin", self.WG_USER2, "Forbidden"],
    ]

    for page in test_pages:
      self.SPDB.url = 'awsaccounts/' + page[0]
      self.set_username(page[2])
      sys.stdout.write("\tGoing to /awsaccounts/" + page[0] + " page (with " + page[1] + " user)... ")
      try:
        self.SPDB.go_home()
        source = self.SPDB.driver.page_source.encode('utf-8').strip()
        if page[3] in source:
          print "SUCCESS! " + page[3] + " appeared for " + page[1] + " user"
        else:
          print source
          print "FAILURE! " + page[3] + " did not appear for " + page[1] + " user"
      except NoSuchElementException, e:
        return None

  def test_314_check_awsaccountrecord(self):
    print "." + inspect.stack()[0][3] + " --> ISHIB-314"

    #
    # get a valid record (page number)
    #
    self.SPDB.url = 'awsaccounts/all'
    self.set_username(self.WG_USER1)
    self.SPDB.go_home()
    source = self.SPDB.driver.page_source.encode('utf-8').strip()
    links = re.findall('.*href.*/awsaccounts/.*View.*', source)

    if not links:
      print "\tFAILURE! No records for admin user"
      return 1

    url_rec = ''
    #
    # get first record in list
    #
    x = links[0].split('"')
    for e in x:
      if 'awsaccounts' in e:
        url_rec = e
        break

    if (len(url_rec) < 1):
      print "\tFAILURE! Cannot extract awsaccount record for admin user"
      return 1

    self.SPDB.url = url_rec
    self.set_username(self.WG_USER1)
    sys.stdout.write("\tGoing to " + url_rec + " page (with admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'Account number' in source:
        print "\tSUCCESS! Account number appeared for admin user"
      else:
        print "\tFAILURE! Account number did not appear for admin user"
    except NoSuchElementException, e:
      return None

    self.SPDB.url = url_rec
    self.set_username(self.WG_USER2)
    sys.stdout.write("\tGoing to " + url_rec + " page (with non-admin user)... ")
    try:
      self.SPDB.go_home()
      source = self.SPDB.driver.page_source.encode('utf-8').strip()
      if 'Forbidden' in source:
        print "SUCCESS! Forbidden message appeared for non-admin user"
      else:
        print "FAILURE! Forbidden message not returned non-admin user"
    except NoSuchElementException, e:
      return None

def suite():
  all_methods = inspect.getmembers(SPDBUnitTest, predicate=inspect.ismethod)
  test_methods = filter(lambda x: re.match(r'^test_\d\d\d', x[0]), all_methods)
  test_method_names = map(lambda x: x[0], test_methods)

  tests = []
  if (SPDBUnitTest.TESTS == 'ALL'):
    tests = sorted(test_method_names)
  else:
    matching_tests = []
    # Split tests into individuals.
    test_numbers = SPDBUnitTest.TESTS.split(',')

    # Add the cleanup test
    test_numbers.append('999')

    # Find the tests corresponding to these.
    for test_number in test_numbers:
      if (not re.match(r'^\d\d\d$', test_number)):
        raise Exception("'" + test_number + "' is not three digits.")

      test_matching = filter(lambda x: 'test_' + test_number in x, test_method_names)
      if (test_matching):
        matching_tests.append(test_matching[0])

    tests = sorted(matching_tests)

  return unittest.TestSuite(map(SPDBUnitTest, tests))


if __name__ == "__main__":
  args = SPDBTesting.parse_args()

  if (args.browser):
    SPDBUnitTest.BROWSER = args.browser
    sys.argv.pop()

  if (args.environment):
    SPDBUnitTest.ENVIRONMENT = args.environment
    sys.argv.pop()

  if (args.tests):
    SPDBUnitTest.TESTS = args.tests
  else:
    SPDBUnitTest.TESTS = 'ALL'

  if (args.verbose):
    SPDBUnitTest.VERBOSE = True
    sys.argv.pop()

  unittest.TextTestRunner(verbosity=1).run(suite())

  if (args.browser == "phantomjs"):
    #
    # PhantomJS may not exit properly
    #
    pid = os.getpid()
    os.system("pkill -KILL -P " + str(pid))
