#!/usr/bin/python
import traceback
from time import sleep
from xml.sax.saxutils import escape, quoteattr
import os, sys, unittest, time, re, random, string, tempfile
from datetime import datetime
from datetime import timedelta
from subprocess import call
import argparse
import inspect
import tempfile
import pprint

# In case selenium module is not installed.
try:
  from selenium import webdriver
  from selenium.webdriver.chrome.options import Options
  from selenium.webdriver.common.by import By
  from selenium.webdriver.common.keys import Keys
  from selenium.webdriver.common.action_chains import ActionChains
  from selenium.webdriver.support.ui import Select
  from selenium.webdriver.support.ui import WebDriverWait
  from selenium.webdriver.support import expected_conditions as EC
  from selenium.common.exceptions import UnexpectedAlertPresentException
  from selenium.common.exceptions import StaleElementReferenceException
  from selenium.common.exceptions import NoAlertPresentException
  from selenium.common.exceptions import NoSuchElementException
  from selenium.webdriver.common.alert import Alert
except ImportError:
  pass

# Note: Remember that WebDriverWait(driver, 30).until(some_method)
# repeatedly calls "some_method(driver)" until that call returns true, or
# else the time runs out.

# If the document is in the "complete" state (i.e., the document has
# completed loading) return true, otherwise return false.
def readystate_complete(driver):
  return driver.execute_script("return document.readyState") == "complete"

class SPDBTesting(object):

  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # Set up the webdriver object. To access it, use "self.driver".
  def __init__(self, environment, browser, username = 'unknown', verbose = False):

    self.environment = environment.lower()
    self.browser     = browser
    self.username    = username
    self.password    = 'spdb-quality-assurance'

    self.verbose     = verbose
    self.pprint      = pprint.PrettyPrinter(indent=4)

    if (browser == 'chrome'):
      if (sys.platform.lower() == 'win32'):
        chromedriver = "D:\\bin\\chromedriver.exe"
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chromedriver)
      else:
        chromedriver = "/usr/bin/chromedriver"
        chrome_options = Options()
        chrome_options.add_argument("--disable-infobars")
        chrome_options.add_argument("--headless")
        os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome(chrome_options=chrome_options)
    elif (browser == 'firefox'):
      self.driver = webdriver.Firefox()
    elif (browser == 'phantomjs'):
      self.driver = webdriver.PhantomJS(service_args=['--ignore-ssl-errors=true'])
      self.driver.set_window_size(1120, 550)
    else:
      raise ValueError("unknown browser " + browser)

    self.fake_wait   = True
    self.test_string = None
    self.driver.maximize_window()
    self.driver.implicitly_wait(30)
    self.use_basic_auth = True

    self.hostname = 'spdb-' + environment + '.stanford.edu'

    # Set the base url
    self.base_url = None
    self.set_base_url()

    self.verificationErrors = []
    self.accept_next_alert = True
    print self.base_url


  def progress(self, message):
    if (self.verbose):
      if (message is None):
        message = 'None'
      print('progress: ' + message)

  def pp(self, object):
    return self.pprint.pformat(object)

  def exit_with_exception(self, message):
    page_source = self.driver.page_source.encode('utf-8').strip()

    fp = open('out.html', 'w')
    fp.write(page_source)
    fp.close()

    self.driver.save_screenshot('out.png');

    self.driver.quit();
    raise Exception(message + "; see screen shot in out.png and page source is in out.html")


  # When running a workgroup membership test we need to have
  # a real user (e.g., "adamhl", "cheroux", "vivienwu", etc.). Note
  # that when we set the username the url changes as well.
  def reset_username(self, user_name):
    self.username = user_name
    self.set_base_url()

  # Set the url attribute. This will depend on the authentication
  # mechanism and the user
  def set_base_url(self):
    if (self.use_basic_auth):
      url = "https://{}:{}@{}/spconfigs".format(self.username, self.password, self.hostname)
    else:
      url = "https://{}/spconfigs".format(self.hostname)

    self.base_url = url
    return self.base_url

  def go_home(self):
    return self.driver.get(self.base_url)

  def go_all(self):
    return self.driver.get(self.base_url + '/all')

  # Get the HTML that comprises the red error text (flash) box at the top
  # of the screen. If there is no such box return None.
  def get_red_error_text(self):
    box_id = 'error_explanation'
    error_box_element = None
    try:
      error_box_element = self.driver.find_element_by_id(box_id)
    except NoSuchElementException, e:
      # No such element, so return None.
      return None

    return error_box_element.get_attribute('innerHTML')

  # Use send_keys on an element but only chunk_size characters at a time.
  def send_keys_slow(self, element, text, chunk_size=128):
    while (len(text) > 0):
      current_chunk = text[0:chunk_size]
      element.send_keys(current_chunk)
      text = text[chunk_size:]

  def get_element_by_id(self, id):
    return self.driver.find_element_by_id(id)

  # Go to the "Add New Service Provider" page.
  def go_to_add_new(self):
    driver = self.driver
    self.go_home()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("button_spconfig_new").click()
    WebDriverWait(driver, 30).until(readystate_complete)

  # Submits the metadata but does not return anything.
  def test_metadata(self, metadata_string):
    self.go_to_add_new()

    driver = self.driver
    time.sleep(5)

    # We require at least one workgroup
    self.add_workgroup_on_edit_form(self.WG1 + '|0')

    driver.find_element_by_id("spconfig_metadata").clear()
    element = driver.find_element_by_id("spconfig_metadata")
    self.send_keys_slow(element, metadata_string)

    driver.find_element_by_id("spconfig_contact_email").clear()
    driver.find_element_by_id("spconfig_contact_email").send_keys("as-acs-linux-teaam@lists.stanford.edu")

    driver.find_element_by_link_text("Submit Metadata").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    if (self.fake_wait):
      time.sleep(10)

    return

  # Use the search function to find one or more SP configs.
  def search_for_sp(self, search_text):
    driver = self.driver

    self.go_home()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("search").clear()
    driver.find_element_by_id("search").send_keys(search_text)
    driver.find_element_by_id("button_spconfig_search").click()
    WebDriverWait(driver, 30).until(readystate_complete)

  # Assuming we are on the view page and there is only ONE SP configuration listed,
  # click the update icon.
  def click_update_icon(self):
    try:
      update_link = self.driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]")
    except:
      self.exit_with_exception("could not find update icon using css selector")

    update_link.click()

  # If we are on the edit form, add the supplied workgroup
  # Expects workgroup to be a string of the form 'workgroup_name|#' where
  # # is an integer.
  def add_workgroup_on_edit_form(self, workgroup):
    temp_name      = workgroup.split("|")
    workgroup_name = temp_name[0]
    workgroup_id   = int(temp_name[1])

    driver = self.driver
    driver.find_element_by_css_selector("img[alt=\"Add workgroup\"]").click()

    element_id     = None
    input_elements = driver.find_elements_by_xpath("//*[contains(@id, 'spconfig_groups_attributes_')]")
    self.progress('add_workgroup input_elements')
    self.progress(self.pp(input_elements))
    if (not input_elements):
      self.exit_with_exception("could not find add workgroup link")

    for element in input_elements:
      element_name = element.get_attribute('name')
      if ("workgroup" in element_name):
        element_id = element.get_attribute('id')
        if (workgroup_id == 0):
          break

    if (element_id is None):
      self.exit_with_exception("could not find add workgroup element")

    driver.find_element_by_id(element_id).clear()
    driver.find_element_by_id(element_id).send_keys(workgroup_name)
    time.sleep(2);

    return

  # Add a workgroup to the SP configuration with the provided
  # entityID. Returns the contents of the owning workgroup element (or
  # None if there is no such element).
  def add_workgroup(self, workgroup, entityID):
    driver = self.driver
    self.search_for_sp(entityID)

    update_link = driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]")
    self.progress('update_link')
    self.progress(self.pp(update_link))

    if (not update_link):
      self.exit_with_exception("could not find update icon using css selector")

    update_link.click()
    WebDriverWait(driver, 30).until(readystate_complete)


    # Add the workgroup
    self.add_workgroup_on_edit_form(workgroup)

    driver.find_element_by_link_text("Save Changes").click()
    WebDriverWait(driver, 30).until(readystate_complete)
    time.sleep(2);

    try:
      owning_workgroups = driver.find_element_by_id('owning_workgroups')
    except NoSuchElementException, e:
      return None

    return owning_workgroups.get_attribute('innerHTML')

  def read_metadata_file(self, filename):
    fh = open(filename, 'r')
    string = fh.read()
    fh.close()
    return string

  @staticmethod
  def get_entityID_from_metadata(metadata):
    rx = re.compile(r'entityID\s*=\s*"([^"]*)"')
    match = re.search(rx, metadata)
    if (match):
      return match.group(1)
    else:
      return None

  @staticmethod
  def make_subject(domain):
    subject_skel='/C=US/ST=California/O=Stanford University/localityName=Palo Alto/commonName=DOMAIN/organizationalUnitName=Administrative Systems/emailAddress=as-acs-linux-team@lists.stanford.edu'

    return subject_skel.replace('DOMAIN', domain)

  @staticmethod
  def create_cert(subject_name, days):
    keyfile = tempfile.NamedTemporaryFile()
    pemfile = tempfile.NamedTemporaryFile()

    subject = SPDBTesting.make_subject(subject_name)
    command = ['openssl',
               'req',
               '-x509',
               '-nodes',
               '-newkey', 'rsa:2048',
               '-keyout', keyfile.name,
               '-out',    pemfile.name,
               '-days',   days,
               '-subj',   subject
               ]
    #print command
    call(command)

    # Get the pem file
    fh = open(pemfile.name, 'r')
    pem_string = fh.read()
    fh.close()

    # Remove the '-----BEGIN CERTIFICATE-----' and
    # '-----END CERTIFICATE-----' strings.
    pem_string = string.replace(pem_string, '-----BEGIN CERTIFICATE-----', '')
    pem_string = string.replace(pem_string, '-----END CERTIFICATE-----', '')
    pem_string = string.strip(pem_string)

    # We can now delete the temporary files.
    keyfile.close()
    pemfile.close()

    return pem_string

  @staticmethod
  def generate_random_domain():
    hostname = ''.join(random.choice(string.ascii_letters) for x in range(8))
    return hostname + '.stanford.edu'

  @staticmethod
  def generate_metadata(domain, cert1 = None, cert2 = None, use_signature = False):

    cert_subject = SPDBTesting.make_subject(domain)

    if (cert1 is None):
      metadata_template = 'meta0.xml'
    else:
      if (cert2 is None):
        metadata_template = 'meta1.xml'
      else:
        metadata_template = 'meta2.xml'

    fh = open(metadata_template, 'r')
    template = fh.read()

    domain_id = domain.replace('/', '')

    # Note that RANDOM1 is the ID attribute hile RANDOM2 is the entityID
    # attribute.
    template = re.sub(r'RANDOM1',  domain_id,    template)
    template = re.sub(r'RANDOM2',  domain,       template)

    if (cert1 is not None):
      template = re.sub(r'CERT1',    cert1,        template)
      template = re.sub(r'SUBJECT1', cert_subject, template)

    if (cert2 is not None):
      template = re.sub(r'CERT2',    cert2,        template)
      template = re.sub(r'SUBJECT1', cert_subject, template)

    # Add signature (if use_signature is true)
    if (use_signature):
      with open('signature.xml','r') as f:
        signature_string = f.read()
    else:
      signature_string = ''

    template = re.sub(r'PUT_SIGNATURE_HERE', signature_string, template)

    return template

  @staticmethod
  def parse_args():
    parser = argparse.ArgumentParser(description='Run Selenium tests')
    parser.add_argument('--environment', help='SPDB environment (PROD, STAGE, UAT, DEV)',
                        )
    parser.add_argument('--browser', help='browser model',
                        choices=['chrome', 'firefox', 'phantomjs'])
    parser.add_argument('--entityID', help='explicitly specify the entityID'
                        )
    parser.add_argument('--generate-metadata', action='store_true',
                        help='generate metadata files only'
                        )
    parser.add_argument('--use-metadata-files', action='store_true',
                        help='use existing metadata files'
                        )

    parser.add_argument('--tests', help='comma-separated list of tests to run (or ALL)'
                        )

    parser.add_argument('--verbose', action='store_true',
                        help='be verbose'
                        )
    return parser.parse_args()

  @staticmethod
  def generate_metadata_file(filename, domain, cert1 = None, cert2 = None, use_signature = False):
    template = SPDBTesting.generate_metadata(domain, cert1, cert2, use_signature)
    with open(filename, "w") as text_file:

      if (filename == 'test_validUntil_expired.xml'):
        _template_ = template.replace('2050-01-01', '2000-01-01')
      elif (filename == 'test_unprefixed_acs_metadata.xml'):
        # _template_ = template.replace('md:AssertionConsumerService', 'AssertionConsumerService')
        _template_ = template.replace('md:', '').replace(':md', '')
      else:
        _template_ = template

      text_file.write(_template_)

    print "generated file " + filename
    return


  # Generates these metadata files in the current directory:
  # * test_single_valid_cert_metadata.xml
  # * test_single_expired_cert_metadata.xml
  # * test_two_expired_cert_metadata.xml
  # * test_one_expired_one_valid_cert_metadata.xml
  #
  # We do this for those platforms that do not have the openssl executable
  # installed.
  #
  # Note that ALL the metadata files have the SAME entityID.
  @staticmethod
  def generate_metadata_files(domain):
    valid_cert1   = SPDBTesting.create_cert(domain, '3650')
    valid_cert2   = SPDBTesting.create_cert(domain, '3650')
    expired_cert1 = SPDBTesting.create_cert(domain, '-1')
    expired_cert2 = SPDBTesting.create_cert(domain, '-1')
    non_self_signed_cert1 =  open('incommon-usertrust-2024.pem', 'r').read()

    filename_to_template = {}

    SPDBTesting.generate_metadata_file('test_single_valid_cert_metadata.xml', domain, valid_cert1)

    SPDBTesting.generate_metadata_file('test_validUntil_expired.xml', domain, valid_cert1)

    SPDBTesting.generate_metadata_file('test_incommon_entity_id.xml', 'smithsonian.idm.oclc.org/shibboleth', valid_cert1)

    SPDBTesting.generate_metadata_file('test_single_expired_cert_metadata.xml', domain, expired_cert1)

    SPDBTesting.generate_metadata_file('test_no_cert_metadata.xml', domain)

    SPDBTesting.generate_metadata_file('test_two_expired_cert_metadata.xml', domain, expired_cert1, expired_cert2)

    SPDBTesting.generate_metadata_file('test_one_expired_one_valid_cert_metadata.xml', domain, expired_cert1, valid_cert1)

    SPDBTesting.generate_metadata_file('test_unprefixed_acs_metadata.xml', domain, expired_cert1, valid_cert1)

    SPDBTesting.generate_metadata_file('test_non_self_signed_cert.xml', domain, non_self_signed_cert1)

    SPDBTesting.generate_metadata_file('test_contains_signature.xml',
                                       domain,
                                       non_self_signed_cert1,
                                       use_signature = True)

    # The bad metadata file
    fp = open('test_single_invalid_metadata.xml', 'w')
    fp.write('bad metadata')
    fp.close()


  @staticmethod
  def assertRegexpMatches(ut, text, expected_regexp, msg=None):
    try:
      unittest.TestCase.assertRegexpMatches(ut, text, expected_regexp, msg)
    except Exception, e:
      # print text
      raise ut.failureException(msg or "Regexp \"" + expected_regexp + "\" didn't match")

  # Read the metadata from the file <filename> and return any text from
  # the red error box (might return None if no error text).
  def testMetadataWrapper(self, filename):
    # Get the metadata.
    metadata = self.read_metadata_file(filename)
    # self.entityID = self.SPDB.get_entityID_from_metadata(metadata)

    # Test it:
    self.test_metadata(metadata)

    return self.get_red_error_text()

  # Find the spconfig matching entityID and delete it.
  # Returns true if actually deleted, false otherwise.
  def deleteSPConfig(self, entityID, browser):
    self.search_for_sp(entityID)

    driver = self.driver

    if (browser == 'phantomjs'):
      # added new hidden link element without confirmation
      element = driver.find_element_by_id ("delete_hidden")
      if (not element):
        self.exit_with_exception("[phantomjs] cannot find delete_hidden element")

      element.click()
    else:
      # "visual" browsers use the alert pop-up
      driver.find_element_by_css_selector("img[alt=\"Deleteicon enabled\"]").click()
      time.sleep(1)
      Alert(driver).accept()

    self.search_for_sp(entityID)
    page_source = driver.page_source.encode('utf-8').strip()
    if (re.match(r'.*0\s*Service\s*Provider\s*Configuration.*', page_source, re.S|re.M)):
      return True
    else:
      self.exit_with_exception("page cannot find delete_hidden element")


    #time.sleep(5)
    #
    # fp = open('/tmp/page.out', 'w')
    # fp.write(driver.page_source.encode('utf-8').strip())
    # fp.close()


class SPDBUnitTest(unittest.TestCase):

  ENVIRONMENT        = 'DEV'
  BROWSER            = 'chrome'
  GENERATE_METADATA  = False
  USE_METADATA_FILES = False
  SPDB               = None
  USERNAME           = 'unknown'
  TESTS              = 'ALL'
  VERBOSE            = False

  # Workgroup settings
  WG1 = 'as-fss:tech'
  WG2 = 'as-fss:dmrteam'

  # WG_USER1 belongs to workgroup WG1. Note that WG_USER1 is an admin, so
  # be sure to run some of the tests as a non-admin.
  WG_USER1 = 'cheroux'

  # WG_USER2 does NOT belong to workgroup WG1. WG_USER2 is NOT an admin.
  WG_USER2 = 'jheroux'

  # WG_USER3 belongs to workgroup WG1.
  WG_USER3 = 'rraman'

  # ADMIN users
  WG_ADMIN   = 'cheroux'

  # REGULAR user
  WG_REGULAR = 'jheroux'

  @classmethod
  def setUpClass(cls):
    # Get comand line args

    if (cls.USE_METADATA_FILES):
      print "using existing metadata files..."
    else:
      print "generating metadata files..."
      domain = SPDBTesting.generate_random_domain()
      print "generating metadata files for domain '" + domain + "'"
      SPDBTesting.generate_metadata_files(domain)
      print "Your entityID is " + domain

    print "running in environment " + cls.ENVIRONMENT
    cls.SPDB = SPDBTesting(environment = cls.ENVIRONMENT,
                           browser     = cls.BROWSER,
                           verbose     = cls.VERBOSE,
    )

  def setUp(self):
    pass

  def progress(self, message):
    if (self.VERBOSE):
      print('progress: ' + message)

  # Call this method before running any workgroup test.
  def set_username(self, user_name):
    self.SPDB.reset_username(user_name)
    return

  def get_page_source(self):
    return self.SPDB.driver.page_source.encode('utf-8').strip()

  # ignore_if_missing: set to True if you want to continue without an
  # error in the event that the entityid does not exist.
  def delete_config(self, filename, ignore_if_missing = False):
    metadata_string = self.SPDB.read_metadata_file(filename)
    entityID        = SPDBTesting.get_entityID_from_metadata(metadata_string)

    if (ignore_if_missing):
      try:
        delete_result = self.SPDB.deleteSPConfig(entityID, self.BROWSER)
        self.assertTrue(delete_result, 'deleted SP config')
      except Exception, e:
        pass
    else:
      delete_result = self.SPDB.deleteSPConfig(entityID, self.BROWSER)
      self.assertTrue(delete_result, 'deleted SP config')

  #
  # test general login page
  #
  #  def test_001_login(self):
  #    driver = self.spdb.driver
  #    driver.set_page_load_timeout(15)
  #    self.spdb.go_home()
  #    WebDriverWait(driver, 30).until(readystate_complete)
  #    self.assertRegexpMatches(driver.page_source, 'Add New Service Provider')

  # All the 1XX tests should be run as WG_USER1. This way, things will be
  # set up properly for the workgroup tests.
  def test_101_fail_to_create_with_invalid_metadata(self):
    print "." + inspect.stack()[0][3]
    expected_text = 'Metadata is not valid'
    self.set_username(self.WG_USER1)

    error_text = self.SPDB.testMetadataWrapper('test_single_invalid_metadata.xml')
    SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  # We no longer care about expired certificates, so we comment out this test.
  #def test_102_fail_to_create_with_single_expired_cert(self):
  #  print inspect.stack()[0][3]
  #  expected_text = 'No certificate found'
  #  self.set_username(self.WG_USER1)

  #  error_text = self.SPDB.testMetadataWrapper('test_single_expired_cert_metadata.xml')

  #  SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  def test_103_fail_to_create_with_no_cert(self):
    print inspect.stack()[0][3]
    #
    # expected_text = 'Metadata must contain at least 1 certificate'
    #
    expected_text = 'Metadata must contain at least one certificate for encryption'
    self.set_username(self.WG_USER1)

    error_text = self.SPDB.testMetadataWrapper('test_no_cert_metadata.xml')
    SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  # We no longer care about expired certificates, so we comment out this test.
#  def test_104_fail_to_create_with_two_expired_certs(self):
#    print inspect.stack()[0][3]
#    expected_text = 'No certificate found'
#    self.set_username(self.WG_USER1)
#
#    error_text = self.SPDB.testMetadataWrapper('test_two_expired_cert_metadata.xml')
#    SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  def test_105_fail_to_create_using_incommon_entity_id(self):
    print inspect.stack()[0][3]
    expected_text = 'metadata is already known'
    #expected_text = 'This entityID is already being used by another configuration'
    self.set_username(self.WG_USER1)

    error_text = self.SPDB.testMetadataWrapper('test_incommon_entity_id.xml')
    SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  # Attempt to create a record with an expired validUntil
  # attribute.
  def test_106_fail_to_create_validUntil_expired(self):
    print inspect.stack()[0][3]
    expected_text = 'validUntil is in the past'
    self.set_username(self.WG_USER1)

    error_text = self.SPDB.testMetadataWrapper('test_validUntil_expired.xml')

  def test_107_at_least_one_encryption_cert_required(self):
    print inspect.stack()[0][3]
    expected_text = 'Metadata must contain at least one certificate for encryption'
    self.set_username(self.WG_USER1)

    #
    # copy valid metadata containing one cert and change <md:KeyDescriptor use="signing">
    #
    xml_file_source = 'test_single_valid_cert_metadata.xml'
    xml_file_copy = 'test_single_valid_cert_signing_only.xml'

    fh = open(xml_file_source, 'r')
    xml_string = fh.read()
    fh.close()

    fh = open(xml_file_copy, 'w')
    xml_string = string.replace(xml_string, '<md:KeyDescriptor>', '<md:KeyDescriptor use="signing">')
    fh.write(xml_string)
    fh.close()

    error_text = self.SPDB.testMetadataWrapper(xml_file_copy)
    SPDBTesting.assertRegexpMatches(self, error_text, expected_text)

  ## All "create"'s that follow should successfully create the SP
  ## configuration and successfully delete.

  # We no longer care about expired certificates, so we comment out this test.
#  def test_120_create_with_one_expired_one_valid_cert(self):
#    print inspect.stack()[0][3]
#    self.set_username(self.WG_USER1)
#    expected_text = 'successfully created'
#    xml_file = 'test_one_expired_one_valid_cert_metadata.xml'
#
#    # Create
#    self.SPDB.testMetadataWrapper(xml_file)
#    page_source = self.get_page_source()
#    SPDBTesting.assertRegexpMatches(self, page_source, expected_text)
#
#    # Delete
#    self.delete_config(xml_file)

  def test_122_create_with_valid_metadata_and_cert(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text = 'successfully created'
    xml_file      = 'test_single_valid_cert_metadata.xml'

    # Test 1. Create
    self.SPDB.testMetadataWrapper(xml_file)
    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text)

    # Test 2. Make sure there are no signatures.
    metadata_string = self.SPDB.read_metadata_file(xml_file)
    entityID        = SPDBTesting.get_entityID_from_metadata(metadata_string)
    self.SPDB.search_for_sp(entityID)

    driver = self.SPDB.driver

    self.SPDB.click_update_icon()
    # driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    page_source = self.get_page_source()
    self.assertFalse(re.search('Signature', page_source),
                     'Metadata <Signature> tag was found and should not exist')

    # Delete
    self.delete_config(xml_file)

  def test_123_create_sp_with_not_self_signed_cert(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text1 = 'successfully created'
    expected_text2 = 'Warning: certificate is not self-signed'
    xml_file       = 'test_non_self_signed_cert.xml'

    # Create.
    self.SPDB.testMetadataWrapper(xml_file)

    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text1)
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text2)

    # Delete
    self.delete_config(xml_file)

  # def test_124_metadata_does_not_contain_signatures(self):
  #   print inspect.stack()[0][3]
  #   text = 'Signature'
  #
  #     self.set_username(self.WG_USER1)
  #
  #     metadata_string = self.SPDB.read_metadata_file('test_single_valid_cert_metadata.xml')
  #     entityID = SPDBTesting.get_entityID_from_metadata(metadata_string)
  #     self.SPDB.search_for_sp(entityID)
  #
  #     driver = self.SPDB.driver
  #
  #     driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]").click()
  #     WebDriverWait(driver, 30).until(readystate_complete)
  #
  #     page_source = self.get_page_source()
  #     self.assertFalse(re.search(text, page_source), 'Metadata <Signature> tag was found and should not exist')


  # Take an existing configuration that has an expired cert and attempt to
  # update one of the non-metadata fields. The update should succeed. This
  # is because we don't want stop people from updating their non-metadata
  # fields if their metadata is invalid.
  #
  # For this test to work there must ALREADY be a configuration in the
  # database with entityID 'config-with-expired-certs' that has an expired
  # certificate. To get this into the database will require some fiddling.
  def test_126_update_metadata_comments_with_expired_certs(self):
    print inspect.stack()[0][3]
    expected_text = 'successfully updated'
    random_str = ''.join([random.choice(string.ascii_letters + string.digits) for n in xrange(32)])

    self.set_username(self.WG_USER1)

    driver = self.SPDB.driver

    entityID = 'config-with-expired-certs'
    self.SPDB.search_for_sp(entityID)

    self.SPDB.click_update_icon()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("spconfig_comments").clear()
    element = driver.find_element_by_id("spconfig_comments")

    self.SPDB.send_keys_slow(element, "Do NOT delete this configuration! We need it for automated regression testing. [ New random text = " + random_str + " ]")

    time.sleep(2);
    driver.find_element_by_link_text("Save Changes").click()
    WebDriverWait(driver, 30).until(readystate_complete)
    time.sleep(2);

    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text)

  # 1. First create a regular record.
  # 2. Update its metadata with metadata that is unprefixed.
  def test_130_update_sp_with_unprefixed_assertion(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text_created = 'successfully created'
    expected_text_updated = 'successfully updated'
    xml_file1             = 'test_one_expired_one_valid_cert_metadata.xml'
    xml_file2             = 'test_unprefixed_acs_metadata.xml'

    # 1. Create
    self.SPDB.testMetadataWrapper(xml_file1)
    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text_created)

    # 2. Find the config and update it.
    metadata_string = self.SPDB.read_metadata_file(xml_file2)
    entityID        = SPDBTesting.get_entityID_from_metadata(metadata_string)
    self.SPDB.search_for_sp(entityID)

    driver = self.SPDB.driver

    driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    driver.find_element_by_id("spconfig_metadata").clear()
    element = driver.find_element_by_id("spconfig_metadata")
    self.SPDB.send_keys_slow(element, metadata_string)

    time.sleep(2);
    driver.find_element_by_link_text("Save Changes").click()
    WebDriverWait(driver, 30).until(readystate_complete)
    time.sleep(2);

    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text_updated)

    # Delete
    self.delete_config(xml_file1)

  def test_140_verify_signature_stripped(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    xml_file = 'test_contains_signature.xml'
    expected_text_created = 'successfully created'

    # 1. Create
    self.SPDB.testMetadataWrapper(xml_file)
    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text_created)

    # Verify that Signature is NOT there.
    self.assertNotRegexpMatches(page_source, r'Signature',
                                'Signature has been removed')

    # Delete
    self.delete_config(xml_file)


  # For the 2XX tests we need a configuration already there.
  def test_200_prepare_sp_for_workgroup_tests(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text = 'successfully created'
    xml_file      = 'test_single_valid_cert_metadata.xml'

    self.SPDB.testMetadataWrapper(xml_file)
    page_source = self.get_page_source()
    SPDBTesting.assertRegexpMatches(self, page_source, expected_text)

    # Don't delete!
    pass

  # WG_USER1: Test that WG_USER1 can add the workgroup WG1
  def test_210_add_workgroup1(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text = self.WG1

    metadata = self.SPDB.read_metadata_file('test_single_valid_cert_metadata.xml')
    entityID = SPDBTesting.get_entityID_from_metadata(metadata)

    self.SPDB.go_home()
    result = self.SPDB.add_workgroup(self.WG1 + '|0', entityID)
    self.assertIn(expected_text, result,
                  'Workgroup as-fss:tech not added')

  # WG_USER1: Test that WG_USER1 can add a second workgroup (WG2)
  def test_211_add_workgroup2(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text = self.WG2

    metadata = self.SPDB.read_metadata_file('test_single_valid_cert_metadata.xml')
    entityID = SPDBTesting.get_entityID_from_metadata(metadata)

    self.SPDB.go_home()
    result = self.SPDB.add_workgroup(self.WG2 + '|1', entityID)
    self.assertIn(expected_text, result,
                  'Workgroup as-fss:dmrteam not added')


  # WG_USER1: Test that WG_USER1 (the owner) cannot add a non-existent (bogus) workgroup.
  def test_220_add_bogus_workgroup(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER1)
    expected_text = 'not a valid workgroup'

    metadata = self.SPDB.read_metadata_file('test_single_valid_cert_metadata.xml')
    entityID = SPDBTesting.get_entityID_from_metadata(metadata)

    self.SPDB.go_home()
    self.SPDB.add_workgroup('bogus_workgroup|2', entityID)

    # We should get an error:
    error_text = self.SPDB.get_red_error_text()
    self.assertIn(expected_text, error_text,
                  'Workgroup bogus_workgroup should display the message - not a valid workgroup')


  # WG_USER2: Test that WG_USER2 who is not a member of either workgroup cannot edit the record.
  def test_301_select_sp_non_user(self):
    print inspect.stack()[0][3]
    self.set_username(self.WG_USER2)

    metadata = self.SPDB.read_metadata_file('test_single_valid_cert_metadata.xml')
    entityID = SPDBTesting.get_entityID_from_metadata(metadata)
    self.SPDB.search_for_sp(entityID)
    driver = self.SPDB.driver

    driver.find_element_by_css_selector("img[alt=\"Updateicon enabled\"]").click()
    WebDriverWait(driver, 30).until(readystate_complete)

    try:
      element = driver.find_element_by_id("spconfig_metadata")
    except NoSuchElementException:
      None

  # WG_USER2: Test that WG_USER2, who is not supposed to be an admin, is,
  # in fact, not an admin.
  def test_302_test_non_admin(self):
    self.set_username(self.WG_USER2)
    print inspect.stack()[0][3]
    self.SPDB.go_all()
    page_source = self.get_page_source()
    self.assertNotRegexpMatches(page_source, r'ADMIN',
                                'non-admin user does not show the ADMIN text')

  # WG_USER2: Test that WG_USER2, who is not an admin, can see all the
  # fields they should be able to see.
  def test_303_test_fields_appear(self):
    self.set_username(self.WG_USER2)
    print inspect.stack()[0][3]
    self.SPDB.go_to_add_new()

    page_source = self.get_page_source()

    self.assertRegexpMatches(page_source, r'id=.spconfig_contact_email.',
                                'contact field appears')

    self.assertRegexpMatches(page_source, r'id=.spconfig_login_url.',
                                'login URL field appears')

    self.assertRegexpMatches(page_source, r'id=.spconfig_comments.',
                                'comments field appears')

    self.assertNotRegexpMatches(page_source, r'ZQQZQ',
                               'bogus text does not appear')


  def test_999_cleanup(self):
    print inspect.stack()[0][3]

    # We used test_single_valid_cert_metadata.xml for the workgroup tests
    # and did not delete. So delete now.
    xml_file = 'test_single_valid_cert_metadata.xml'
    self.delete_config(xml_file, ignore_if_missing = True)

def suite():
  all_methods = inspect.getmembers(SPDBUnitTest, predicate=inspect.ismethod)
  test_methods = filter(lambda x: re.match(r'^test_\d\d\d', x[0]), all_methods)
  test_method_names = map(lambda x: x[0], test_methods)

  tests = []
  if (SPDBUnitTest.TESTS == 'ALL'):
    tests = sorted(test_method_names)
  else:
    matching_tests = []
    # Split tests into individuals.
    test_numbers = SPDBUnitTest.TESTS.split(',')

    # Add the cleanup test
    test_numbers.append('999')

    # Find the tests corresponding to these.
    for test_number in test_numbers:
      if (not re.match(r'^\d\d\d$', test_number)):
        raise Exception("'" + test_number + "' is not three digits.")

      test_matching = filter(lambda x: 'test_' + test_number in x, test_method_names)
      if (test_matching):
        matching_tests.append(test_matching[0])

    tests = sorted(matching_tests)

  #
  # tests = [ 'test_103_fail_to_create_with_no_cert', ];
  #
  # tests = [ 'test_105_fail_to_create_using_incommon_entity_id', ];
  #
  # tests = [
  #         'test_120_create_sp_with_one_expired_one_valid_cert',
  #         'test_122_create_sp_with_valid_metadata_and_cert',
  #         # 'test_123_create_sp_with_not_self_signed_cert',
  #         # 'test_130_update_sp_with_unprefixed_assertion',
  #         # 'test_302_test_non_admin',
  #        ]

  print "Will run these tests:"
  for test in tests:
    print ' - ' + test

  return unittest.TestSuite(map(SPDBUnitTest, tests))


if __name__ == "__main__":
  args = SPDBTesting.parse_args()

  if (args.generate_metadata):
    SPDBUnitTest.GENERATE_METADATA = True
    sys.argv.pop()

  if (args.use_metadata_files):
    SPDBUnitTest.USE_METADATA_FILES = True
    sys.argv.pop()

  if (args.browser):
    SPDBUnitTest.BROWSER = args.browser
    sys.argv.pop()

  if (args.environment):
    SPDBUnitTest.ENVIRONMENT = args.environment
    sys.argv.pop()

  if (args.tests):
    SPDBUnitTest.TESTS = args.tests
  else:
    SPDBUnitTest.TESTS = 'ALL'

  if (args.verbose):
    SPDBUnitTest.VERBOSE = True
    sys.argv.pop()

  if (args.generate_metadata):
    domain = SPDBTesting.generate_random_domain()
    print "generating metadata files for domain '" + domain + "'"
    SPDBTesting.generate_metadata_files(domain)
    print "Your entityID is " + domain
    sys.exit(0)
  else:
    print "About to run TestRunner"
    #
    # unittest.main(exit=False)
    #
    unittest.TextTestRunner(verbosity=1).run(suite())
    #

  if (args.browser == "phantomjs"):
    #
    # PhantomJS may not exit properly
    #
    pid = os.getpid()
    os.system("pkill -KILL -P " + str(pid))
